@@ -365,7 +365,7 @@ def uploadlfiles(ui, rsrc, rdst, files):
     at = 0
     ui.debug("sending statlfile command for %d largefiles\n" % len(files))
     retval = store.exists(files)
-    files = filter(lambda h: not retval[h], files)
+    files = [h for h in files if not retval[h]]
     ui.debug("%d largefiles need to be uploaded\n" % len(files))
 
     for hash in files:
@@ -589,7 +589,7 @@ def lfpull(ui, repo, source="default", **opts):
 
     numcached = 0
     for rev in revs:
-        ui.note(_('pulling largefiles for revision %s\n') % rev)
+        ui.note(_('pulling largefiles for revision %d\n') % rev)
         (cached, missing) = cachelfiles(ui, repo, rev)
         numcached += len(cached)
     ui.status(_("%d largefiles cached\n") % numcached)
